"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_process_1 = require("@ionic/utils-process");
const lodash = require("lodash");
const color_1 = require("../lib/color");
const command_1 = require("../lib/command");
const errors_1 = require("../lib/errors");
const serve_1 = require("../lib/serve");
class ServeCommand extends command_1.Command {
    async getMetadata() {
        let groups = [];
        const options = [
            ...serve_1.COMMON_SERVE_COMMAND_OPTIONS,
            {
                name: 'lab-host',
                summary: 'Use specific address for Ionic Lab server',
                default: 'localhost',
                groups: ["advanced" /* ADVANCED */],
                spec: { value: 'host' },
            },
            {
                name: 'lab-port',
                summary: 'Use specific port for Ionic Lab server',
                default: serve_1.DEFAULT_LAB_PORT.toString(),
                groups: ["advanced" /* ADVANCED */],
                spec: { value: 'port' },
            },
            {
                name: 'open',
                summary: 'Do not open a browser window',
                type: Boolean,
                default: true,
            },
            {
                name: 'browser',
                summary: `Specifies the browser to use (${serve_1.BROWSERS.map(b => color_1.input(b)).join(', ')})`,
                aliases: ['w'],
                groups: ["advanced" /* ADVANCED */],
            },
            {
                name: 'browseroption',
                summary: `Specifies a path to open to (${color_1.input('/#/tab/dash')})`,
                aliases: ['o'],
                groups: ["advanced" /* ADVANCED */],
                spec: { value: 'path' },
            },
            {
                name: 'lab',
                summary: 'Test your apps on multiple platform types in the browser',
                type: Boolean,
                aliases: ['l'],
            },
        ];
        const exampleCommands = ['', '--local', '--lab'];
        const footnotes = [];
        let description = `
Easily spin up a development server which launches in your browser. It watches for changes in your source files and automatically reloads with the updated build.

By default, ${color_1.input('ionic serve')} boots up a development server on all network interfaces and prints the external address(es) on which your app is being served. It also broadcasts your app to the Ionic DevApp on your network. To disable the DevApp and bind to ${color_1.input('localhost')}, use ${color_1.input('--local')}.

Try the ${color_1.input('--lab')} option to see multiple platforms at once.`;
        const runner = this.project && await this.project.getServeRunner();
        if (runner) {
            const libmetadata = await runner.getCommandMetadata();
            groups = libmetadata.groups || [];
            options.push(...libmetadata.options || []);
            description += `\n\n${(libmetadata.description || '').trim()}`;
            footnotes.push(...libmetadata.footnotes || []);
            exampleCommands.push(...libmetadata.exampleCommands || []);
        }
        return {
            name: 'serve',
            type: 'project',
            summary: 'Start a local dev server for app dev/testing',
            description,
            footnotes,
            groups,
            exampleCommands,
            options,
        };
    }
    async preRun(inputs, options, runinfo) {
        const lastpath = lodash.last(runinfo.location.path);
        const alias = lastpath ? lastpath[0] : undefined;
        if (alias === 'lab') {
            options['lab'] = true;
        }
        if (options['nolivereload']) {
            this.env.log.warn(`The ${color_1.input('--nolivereload')} option has been deprecated. Please use ${color_1.input('--no-livereload')}.`);
            options['livereload'] = false;
        }
        if (options['nobrowser']) {
            this.env.log.warn(`The ${color_1.input('--nobrowser')} option has been deprecated. Please use ${color_1.input('--no-open')}.`);
            options['open'] = false;
        }
        if (options['b']) {
            options['open'] = false;
        }
        if (options['noproxy']) {
            this.env.log.warn(`The ${color_1.input('--noproxy')} option has been deprecated. Please use ${color_1.input('--no-proxy')}.`);
            options['proxy'] = false;
        }
        if (options['x']) {
            options['proxy'] = false;
        }
    }
    async run(inputs, options) {
        if (!this.project) {
            throw new errors_1.FatalException(`Cannot run ${color_1.input('ionic serve')} outside a project directory.`);
        }
        // TODO: use runner directly
        await serve_1.serve({ flags: this.env.flags, config: this.env.config, log: this.env.log, prompt: this.env.prompt, shell: this.env.shell, project: this.project }, inputs, options);
        await utils_process_1.sleepForever();
    }
}
exports.ServeCommand = ServeCommand;
class LabCommand extends ServeCommand {
    async getMetadata() {
        const metadata = await super.getMetadata();
        const groups = [...metadata.groups || [], "hidden" /* HIDDEN */];
        const exampleCommands = [...metadata.exampleCommands || []].filter(c => !c.includes('--lab'));
        return {
            ...metadata,
            summary: 'Start Ionic Lab for multi-platform dev/testing',
            description: `
Start an instance of ${color_1.strong('Ionic Lab')}, a tool for developing Ionic apps for multiple platforms at once side-by-side.

${color_1.input('ionic lab')} is just a convenient shortcut for ${color_1.input('ionic serve --lab')}.`,
            groups,
            exampleCommands,
        };
    }
}
exports.LabCommand = LabCommand;
