"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const serve_1 = require("@ionic/cli-utils/lib/serve");
const app_scripts_1 = require("@ionic/cli-utils/lib/ionic-angular/app-scripts");
const utils_1 = require("@ionic/cli-utils/lib/cordova/utils");
exports.CORDOVA_RUN_COMMAND_OPTIONS = [
    {
        name: 'list',
        description: 'List all available Cordova targets',
        type: Boolean,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'livereload',
        description: 'Spin up server to live-reload www files',
        type: Boolean,
        aliases: ['l'],
    },
    {
        name: 'consolelogs',
        description: 'Print out console logs to terminal',
        type: Boolean,
        aliases: ['c'],
    },
    {
        name: 'serverlogs',
        description: 'Print out dev server logs to terminal',
        type: Boolean,
        aliases: ['s'],
        visible: false,
    },
    {
        name: 'address',
        description: 'Use specific address for the dev server',
        default: serve_1.BIND_ALL_ADDRESS,
        advanced: true,
    },
    {
        name: 'port',
        description: 'Use specific port for HTTP',
        default: String(serve_1.DEFAULT_SERVER_PORT),
        aliases: ['p'],
        advanced: true,
    },
    {
        name: 'livereload-port',
        description: 'Use specific port for live-reload',
        default: String(serve_1.DEFAULT_LIVERELOAD_PORT),
        aliases: ['r'],
        advanced: true,
    },
    {
        name: 'dev-logger-port',
        description: 'Use specific port for dev server communication',
        default: String(serve_1.DEFAULT_DEV_LOGGER_PORT),
        advanced: true,
    },
    ...app_scripts_1.APP_SCRIPTS_OPTIONS,
    {
        name: 'debug',
        description: 'Mark as a debug build',
        type: Boolean,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'release',
        description: 'Mark as a release build',
        type: Boolean,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'device',
        description: 'Deploy Cordova build to a device',
        type: Boolean,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'emulator',
        description: 'Deploy Cordova build to an emulator',
        type: Boolean,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'target',
        description: `Deploy Cordova build to a device (use ${chalk_1.default.green('--list')} to see all)`,
        type: String,
        intents: [utils_1.CORDOVA_INTENT],
    },
    {
        name: 'buildConfig',
        description: 'Use the specified Cordova build configuration',
        intents: [utils_1.CORDOVA_INTENT],
        advanced: true,
    },
];
class CordovaCommand extends command_1.Command {
    preRunChecks() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ConfigXml } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/cordova/config'));
            const { prettyPath } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/format'));
            yield utils_1.checkCordova(this.env);
            // Check for www folder
            if (this.env.project.directory) {
                const wwwPath = path.join(this.env.project.directory, 'www');
                const wwwExists = yield fs_1.pathExists(wwwPath); // TODO: hard-coded
                if (!wwwExists) {
                    this.env.tasks.next(`Creating ${chalk_1.default.bold(prettyPath(wwwPath))} directory for you`);
                    yield fs_1.fsMkdir(wwwPath, 0o777);
                    this.env.tasks.end();
                }
            }
            const conf = yield ConfigXml.load(this.env.project.directory);
            conf.resetContentSrc();
            yield conf.save();
        });
    }
    runCordova(argList, _a = {}) {
        var { fatalOnNotFound = false, truncateErrorOutput = 5000 } = _a, options = tslib_1.__rest(_a, ["fatalOnNotFound", "truncateErrorOutput"]);
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ERROR_SHELL_COMMAND_NOT_FOUND } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/shell'));
            const { pkgManagerArgs } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/npm'));
            try {
                return yield this.env.shell.run('cordova', argList, Object.assign({ fatalOnNotFound, truncateErrorOutput }, options));
            }
            catch (e) {
                if (e === ERROR_SHELL_COMMAND_NOT_FOUND) {
                    const cdvInstallArgs = yield pkgManagerArgs(this.env, { pkg: 'cordova', global: true });
                    throw new errors_1.FatalException(`The Cordova CLI was not found on your PATH. Please install Cordova globally:\n` +
                        `${chalk_1.default.green(cdvInstallArgs.join(' '))}\n`);
                }
                if (options.fatalOnError) {
                    this.env.log.nl();
                    this.env.log.error('Cordova encountered an error.\nYou may get more insight by running the Cordova command above directly.\n');
                }
                throw e;
            }
        });
    }
    checkForPlatformInstallation(runPlatform) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (runPlatform) {
                const { getPlatforms, installPlatform } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/cordova/project'));
                const platforms = yield getPlatforms(this.env.project.directory);
                if (!platforms.includes(runPlatform)) {
                    yield installPlatform(this.env, runPlatform);
                }
            }
        });
    }
}
exports.CordovaCommand = CordovaCommand;
class CordovaRunCommand extends CordovaCommand {
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.preRunChecks();
            if (options['list']) {
                const args = utils_1.filterArgumentsForCordova(this.metadata, inputs, options);
                if (!options['device'] && !options['emulator']) {
                    if (args[0] === 'run') {
                        args.push('--device');
                    }
                    else if (args[0] === 'emulate') {
                        args.push('--emulator');
                    }
                }
                args[0] = 'run';
                yield this.runCordova(args, { showExecution: true });
                throw new errors_1.FatalException('', 0);
            }
            if (!inputs[0]) {
                const platform = yield this.env.prompt({
                    type: 'input',
                    name: 'platform',
                    message: `What platform would you like to run (${['android', 'ios'].map(v => chalk_1.default.green(v)).join(', ')}):`,
                });
                inputs[0] = platform.trim();
            }
            yield this.checkForPlatformInstallation(inputs[0]);
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { ConfigXml } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/cordova/config'));
            const { registerShutdownFunction } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/process'));
            if (!options['livereload'] && (options['consolelogs'] || options['serverlogs'])) {
                this.env.log.info(`${chalk_1.default.green('--consolelogs')} or ${chalk_1.default.green('--serverlogs')} detected, using ${chalk_1.default.green('--livereload')}`);
                options['livereload'] = true;
            }
            const isLiveReload = options['livereload'];
            const conf = yield ConfigXml.load(this.env.project.directory);
            registerShutdownFunction(this.env, () => {
                conf.resetContentSrc();
                conf.saveSync();
            });
            if (isLiveReload) {
                const { serve } = yield Promise.resolve().then(() => require('@ionic/cli-utils/commands/serve'));
                const serverDetails = yield serve(this.env, inputs, utils_1.generateBuildOptions(this.metadata, options));
                if (serverDetails.externallyAccessible === false) {
                    const extra = serve_1.LOCAL_ADDRESSES.includes(serverDetails.externalAddress) ? '\nEnsure you have proper port forwarding setup from your device to your computer.' : '';
                    this.env.log.warn(`Your device or emulator may not be able to access ${chalk_1.default.bold(serverDetails.externalAddress)}.${extra}\n\n`);
                }
                conf.writeContentSrc(`${serverDetails.protocol || 'http'}://${serverDetails.externalAddress}:${serverDetails.port}`);
                yield conf.save();
            }
            else {
                const { build } = yield Promise.resolve().then(() => require('@ionic/cli-utils/commands/build'));
                yield build(this.env, inputs, utils_1.generateBuildOptions(this.metadata, options));
            }
            yield this.runCordova(utils_1.filterArgumentsForCordova(this.metadata, inputs, options), { showExecution: true });
            if (!isLiveReload) {
                this.env.log.ok(`Your app has been deployed.\n` +
                    `Did you know you can live-reload changes from your app with ${chalk_1.default.green('--livereload')}?`);
            }
        });
    }
}
exports.CordovaRunCommand = CordovaRunCommand;
