"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const command_1 = require("@ionic/cli-utils/lib/command");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const errors_1 = require("@ionic/cli-utils/lib/errors");
let DaemonCommand = class DaemonCommand extends command_1.Command {
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { prettyPath } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/format'));
            const { createCommServer, processRunning } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/daemon'));
            const { pkgLatestVersion } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/npm'));
            const { findClosestOpenPort } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/network'));
            const { determineDistTag, versionNeedsUpdating } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/plugins'));
            const { registerShutdownFunction } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/process'));
            const updateInterval = Number(options.interval);
            const killExisting = options['kill-existing'];
            const config = yield this.env.config.load();
            if (!config.daemon.updates) {
                throw new errors_1.FatalException('Daemon is disabled in config.');
            }
            const f = yield this.env.daemon.getPid();
            const d = yield this.env.daemon.load();
            d.daemonVersion = this.env.plugins.ionic.meta.version;
            const daemonDistTag = determineDistTag(this.env.plugins.ionic.meta.version);
            let latestIonicVersion = this.env.plugins.ionic.meta.latestVersion;
            if (!latestIonicVersion) {
                latestIonicVersion = yield pkgLatestVersion(this.env, 'ionic', daemonDistTag);
                if (!latestIonicVersion) {
                    throw new errors_1.FatalException('Could not get latest version of ionic.');
                }
                this.env.daemon.populateDistTag(daemonDistTag);
                d.latestVersions[daemonDistTag]['ionic'] = latestIonicVersion;
            }
            yield this.env.daemon.save();
            if (f) {
                this.env.log.info(`Daemon pid file found: ${chalk_1.default.bold(prettyPath(this.env.daemon.pidFilePath))}`);
                if (killExisting) {
                    this.env.log.info(`Killing existing daemon process ${chalk_1.default.bold(String(f))}.`);
                    yield fs_1.fsUnlink(this.env.daemon.pidFilePath);
                    try {
                        process.kill(Number(f));
                    }
                    catch (e) {
                        if (e.code !== 'ESRCH') {
                            throw e;
                        }
                    }
                }
                else if (!processRunning(f)) {
                    this.env.log.info(`Process ${chalk_1.default.bold(String(f))} not found, deleting pid file.`);
                    yield fs_1.fsUnlink(this.env.daemon.pidFilePath);
                }
                else {
                    this.env.log.info('Daemon already running and up-to-date.');
                    return;
                }
            }
            this.env.log.info(`Writing ${chalk_1.default.bold(String(process.pid))} to daemon pid file (${chalk_1.default.bold(prettyPath(this.env.daemon.pidFilePath))}).`);
            yield this.env.daemon.setPid(process.pid);
            const commServerHost = 'localhost';
            const commServerPort = yield findClosestOpenPort(53818, commServerHost);
            this.env.log.info(`Spinning up communication server on port ${chalk_1.default.bold(String(commServerPort))}.`);
            const commServer = yield createCommServer(this.env);
            commServer.listen(commServerPort, commServerHost);
            this.env.log.info(`Writing ${chalk_1.default.bold(String(commServerPort))} to daemon port file (${chalk_1.default.bold(prettyPath(this.env.daemon.portFilePath))}).`);
            yield this.env.daemon.setPort(commServerPort);
            const updateFn = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const config = yield this.env.config.load({ disk: true });
                const f = yield this.env.daemon.getPid();
                const d = yield this.env.daemon.load({ disk: true });
                if (!f) {
                    this.env.log.info(`Daemon shutting down--pid file missing.`);
                    return process.exit();
                }
                else if (f !== process.pid) {
                    this.env.log.info(`Daemon shutting down--mismatch with pid file. (${chalk_1.default.bold(String(f))} vs ${chalk_1.default.bold(String(process.pid))})`);
                    return process.exit();
                }
                else if (!config.daemon.updates) {
                    this.env.log.info(`Daemon shutting down--daemon was disabled.`);
                    return process.exit();
                }
                else if (daemonDistTag !== determineDistTag(config.version)) {
                    this.env.log.info(`Daemon shutting down--dist-tag mismatch. (${chalk_1.default.bold(d.daemonVersion)} vs ${chalk_1.default.bold(config.version)})`);
                    return process.exit();
                }
                else if (yield versionNeedsUpdating(d.daemonVersion, config.version)) {
                    this.env.log.info(`Daemon shutting down--out-of-date. (${chalk_1.default.bold(d.daemonVersion)} vs ${chalk_1.default.bold(config.version)})`);
                    return process.exit();
                }
                for (let distTag in d.latestVersions) {
                    const pkgs = Object.keys(d.latestVersions[distTag]);
                    const pkgUpdates = yield Promise.all(pkgs.map(pkg => pkgLatestVersion(this.env, pkg, distTag)));
                    for (let i in pkgUpdates) {
                        d.latestVersions[distTag][pkgs[i]] = pkgUpdates[i] || '';
                    }
                }
                this.env.log.info('Writing daemon file.');
                yield this.env.daemon.save();
            });
            registerShutdownFunction(this.env, () => {
                const fs = require('fs');
                try {
                    fs.unlinkSync(this.env.daemon.pidFilePath);
                }
                catch (e) {
                    if (e.code !== 'ENOENT') {
                        throw e;
                    }
                }
                try {
                    fs.unlinkSync(this.env.daemon.portFilePath);
                }
                catch (e) {
                    if (e.code !== 'ENOENT') {
                        throw e;
                    }
                }
            });
            const delayMs = 5 * 1000; // wait 5 seconds before doing first check
            const updateIntervalMs = updateInterval * 1000; // check every interval
            setTimeout(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield updateFn();
                setInterval(updateFn, updateIntervalMs);
            }), delayMs);
        });
    }
};
DaemonCommand = tslib_1.__decorate([
    command_1.CommandMetadata({
        name: 'daemon',
        type: 'global',
        description: 'Ionic update checker daemon',
        options: [
            {
                name: 'interval',
                description: 'Interval, in seconds, to check for updates',
                default: '900',
            },
            {
                name: 'kill-existing',
                description: 'If an existing daemon is found, force kill it',
                type: Boolean,
            },
        ],
        visible: false,
    })
], DaemonCommand);
exports.DaemonCommand = DaemonCommand;
