"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const lib_1 = require("@ionic/cli-framework/lib");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const TYPE_CHOICES = ['component', 'directive', 'page', 'pipe', 'provider', 'tabs'];
let GenerateCommand = class GenerateCommand extends command_1.Command {
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const project = yield this.env.project.load();
            if (project.type !== 'ionic-angular') {
                throw new errors_1.FatalException('Generators are only supported in Ionic Angular projects.');
            }
            if (!inputs[0]) {
                const generatorType = yield this.env.prompt({
                    type: 'list',
                    name: 'generatorType',
                    message: 'What would you like to generate:',
                    choices: TYPE_CHOICES,
                });
                inputs[0] = generatorType;
            }
            if (!inputs[1]) {
                const generatorName = yield this.env.prompt({
                    type: 'input',
                    name: 'generatorName',
                    message: 'What should the name be?',
                    validate: v => lib_1.validators.required(v, 'name')
                });
                inputs[1] = generatorName;
            }
            if (!this.env.flags.interactive && inputs[0] === 'tabs') {
                throw new errors_1.FatalException(`Cannot generate tabs without prompts. Run without ${chalk_1.default.green('--no-interactive')}.`);
            }
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [type, name] = inputs;
            const { generate } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/ionic-angular/generate'));
            yield generate({ env: this.env, inputs, options });
            this.env.log.ok(`Generated a ${chalk_1.default.bold(type)}${type === 'tabs' ? ' page' : ''} named ${chalk_1.default.bold(name)}!`);
        });
    }
};
GenerateCommand = tslib_1.__decorate([
    command_1.CommandMetadata({
        name: 'generate',
        type: 'project',
        description: `Generate pipes, components, pages, directives, providers, and tabs ${chalk_1.default.bold(`(ionic-angular >= 3.0.0)`)}`,
        longDescription: `
Automatically create components for your Ionic app.

The given ${chalk_1.default.green('name')} is normalized into an appropriate naming convention. For example, ${chalk_1.default.green('ionic generate page neat')} creates a page by the name of ${chalk_1.default.green('NeatPage')} in ${chalk_1.default.green('src/pages/neat/')}.
  `,
        exampleCommands: [
            '',
            ...TYPE_CHOICES,
            'component foo',
            'page Login',
            'page Detail --no-module',
            'page About --constants',
            'pipe MyFilterPipe'
        ],
        inputs: [
            {
                name: 'type',
                description: `The type of generator (e.g. ${TYPE_CHOICES.map(t => chalk_1.default.green(t)).join(', ')})`,
                validators: [lib_1.contains(TYPE_CHOICES, {})],
            },
            {
                name: 'name',
                description: 'The name of the component being generated',
            }
        ],
        options: [
            {
                name: 'module',
                description: 'Do not generate an NgModule for the component',
                type: Boolean,
                default: true
            },
            {
                name: 'constants',
                description: 'Generate a page constant file for lazy-loaded pages',
                type: Boolean,
                default: false
            }
        ]
    })
], GenerateCommand);
exports.GenerateCommand = GenerateCommand;
